//#if __PTR_WIDTH__ == 32
//   #pragma pack(4)
//#else
//   #pragma pack(8)
//#endif

typedef enum
{
   noError        = 0,
   syntaxError    = 1,
   semanticError  = 2,
   paramError     = 4,
   logicalError   = 8,
   typeError      = 16,
   nanError       = 32,
   singularError  = 64,
   oorError       = 128,
   recursionError = 256,
} ErrorFlags;

typedef enum
{
   rtNone      = 0,
   rtBoolean   = 1,
   rtInteger   = 2,
   rtReal      = 4,
   rtComplex   = 8,
   rtVector    = 16,
   rtString    = 32,
   rtUndefined = 64,
   rtError     = 128,

   rtScalar    = rtBoolean|rtInteger|rtReal,
} ResultTypeFlags;

typedef enum
{
   factor,
   function,
   monadic,
   special,
   exproot,
   muldiv,
   addsub,
   bshift,
   compare,
   band,
   bxor,
   bor,
   land,
   lxor,
   lor,
   equate,
   assign,
   separate
} Precedence;

typedef enum
{
   param      = 0,
   unary      = 1,
   binary     = 2,
   ternary    = 3,
   quaternary = 4,
   quinary    = 5,
   multiple   = 0x7FFFFFFF
} OpKind;

typedef enum
{
   openParent    = 1,
   openComplex   = 2,
   openVector    = 4,
   closeBrackets = 8,
   listSep       = 16,
   unaryOp       = 32,
   binaryOp      = 64,
   aPlus         = 128,
   aMinus        = 256,
   aPoint        = 512,
   anE           = 1024,
   numPlusMinus  = 2048,
   numDigit      = 4096,
   numPoint      = 8192,
   numExponent   = 16384,
   numPercent    = 32768,
   hexNum        = 65536,
   binNum        = 131072,
   other         = 262144,
   numVarFunc    = 524288,
   blank         = 1048576,
   assignment    = 2097152
} SyntaxFlags;

typedef unsigned boolean;

#ifndef false
#define false ((boolean)0)
#endif

#ifndef true
#define true  ((boolean)1)
#endif


typedef struct
{
   ErrorFlags   flag;
   unsigned     begin, count;
   char        *field;
} ErrorRecord;


typedef struct
{
   ResultTypeFlags     f;
   unsigned            n;
   boolean             b;
   long long           i;
   long double         r;
   long double complex z;
   struct ResultType  *v;
   char               *s;
   ErrorRecord         e;
} ResultType;


typedef struct TokenRecord *TokenPtr;
@class Calculator;

@interface CalcObject : BareClass
{
   boolean      isPercent;
   TokenPtr     itsToken;
   Precedence   itsPrecedence;
   OpKind       itsOpKind;
   CalcObject *itsParent;
   Calculator *itsCalculator;
}

- (id)init:(TokenPtr)token :(Precedence)precedence :(OpKind)opKind;

- (boolean)isPercent;
- (void)setIsPercent:(boolean)flag;

- (id)itsParent;
- (void)setItsParent:(CalcObject *)parent;

- (Calculator *)itsCalculator;
- (void)setItsCalculator:(Calculator *)calculator;

- (ResultType)evaluate;

@end


typedef struct TokenRecord
{
   unsigned    tBegin;
   unsigned    tCount;
   SyntaxFlags tCheck;
   char        tLiteral[256];
   CalcObject *tObject;
   TokenPtr    tNext;
} TokenRecord;


@interface CalcFactor : CalcObject
{
   ResultType itsValue;
}

- (ResultType)itsValue;
- (void)setItsValue:(ResultType)value;

- (void)setBooleanValue:(boolean)boolval;
- (void)setIntValue:(long long)integer;
- (void)setRealValue:(long double)real;
- (void)setComplexValue:(long double complex)complx;
- (void)setVectorValue:(struct ResultType *)vector count:(unsigned)n;
- (void)setDescription:(char *)description;
- (void)setError:(ErrorRecord)error;

@end

typedef struct CalcObjectTreeNode *CalcObjectTreePtr;
typedef struct CalcObjectTreeNode
{
   char             *identifier;
   CalcObject       *object;

   CalcObjectTreePtr L, R;
   int               B;
} CalcObjectTreeNode;


#define buffersize 1024 // size in bytes of randomized buffer, must not be less than 128

typedef struct
{
   __m128i  buf[buffersize/sizeof(__m128i)];
   __m128i  key[2];
   uint64_t seed[2];
   int32_t  level;
} CC12Pool;


@interface Calculator : BareClass
{
   boolean            isMaster;
   int                counter0, counter1;
   uint8_t           *randomPoolBaseAddress;
   CC12Pool          *randomPool;
   CalcObjectTreePtr  rootNode;
}

- (id)initWithRootNode:(CalcObjectTreePtr)node;

@end

//#pragma pack()
