#import <stdio.h>
#import <stdlib.h>
#import <stdint.h>
#import <math.h>
#import <complex.h>
#import <tmmintrin.h>
#import <objc/Object.h>

#import "BareClass.h"
#import "CalcObject.h"


@implementation CalcObject

- (id)init:(TokenPtr)token :(Precedence)precedence :(OpKind)opKind
{
   if (self = [self init])
   {
      itsToken      = token;
      itsPrecedence = precedence;
      itsOpKind     = opKind;
   }

   return self;
}

- (boolean)isPercent
{
   return isPercent;
}

- (void)setIsPercent:(boolean)flag
{
   isPercent = flag;
}

- (id)itsParent
{
   return itsParent;
}

- (void)setItsParent:(CalcObject *)parent
{
   itsParent = parent;           // does not own itsParent
}

- (Calculator *)itsCalculator
{
   return itsCalculator;
}

- (void)setItsCalculator:(Calculator *)calculator
{
   itsCalculator = calculator;   // does not own itsCalculator
}

- (ResultType)evaluate
{
   return (ResultType){rtNone, 0, false, 0LL, 0.0L, {0.0L, 0.0L}, NULL, NULL, {noError, 0, 0, NULL}};
}

@end


@implementation CalcFactor

- (ResultType)itsValue
{
   return itsValue;
}

- (void)setItsValue:(ResultType)value
{
   itsValue = value;
}

- (void)setBooleanValue:(boolean)boolval
{
   itsValue = (ResultType){rtBoolean, 0, boolval, 0LL, 0.0L, {0.0L, 0.0L}, NULL, NULL, {noError, 0, 0, NULL}};
}

- (void)setIntValue:(long long)integer
{
   itsValue = (ResultType){rtInteger, 0, false, integer, 0.0L, {0.0L, 0.0L}, NULL, NULL, {noError, 0, 0, NULL}};
}

- (void)setRealValue:(long double)real
{
   itsValue = (ResultType){rtReal, 0, false, 0LL, real, {0.0L, 0.0L}, NULL, NULL, {noError, 0, 0, NULL}};
}

- (void)setComplexValue:(long double complex)complx
{
   itsValue = (ResultType){rtComplex, 0, false, 0LL, 0.0L, complx, NULL, NULL, {noError, 0, 0, NULL}};
}

- (void)setVectorValue:(struct ResultType *)vector count:(unsigned)n
{
   itsValue = (ResultType){rtVector, n, false, 0LL, 0.0L, {0.0L, 0.0L}, vector, NULL, {noError, 0, 0, NULL}};
}

- (void)setDescription:(char *)description
{
   itsValue.s = description;
}

- (void)setError:(ErrorRecord)error
{
   itsValue.e = error;
}

- (ResultType)evaluate
{
   return itsValue;
}

@end


@implementation Calculator

- (id)initWithRootNode:(CalcObjectTreePtr)node
{
   static Calculator *master = nil;

   if (self = [self init])
   {
      rootNode = node;

      if (!master)
      {
         master = self;
         isMaster = YES;
      }
   }

   return self;
}


void releaseCalcObjectsTree(CalcObjectTreePtr node)
{
   if (node)
   {
      releaseCalcObjectsTree(node->R);
      releaseCalcObjectsTree(node->L);
      [node->object release];
   }
}

- (void)dealloc
{
   releaseCalcObjectsTree(rootNode);
   [super dealloc];
}

@end
